@@ -240,9 +240,12 @@
   /dev/null so that fds [0:2] are open (was only doing fds [0:1]) and prevent
   infinite loop if /dev/null cannot be opened.
 
-    Last updated by Richard Gooch   20-JAN-2002: Fixed <action_compat> to
+    Updated by      Richard Gooch   20-JAN-2002: Fixed <action_compat> to
   ignore new compatibility names for IDE devices.
 
+    Last updated by Richard Gooch   14-FEB-2002: Do not re-read config file if
+  signals other than SIGHUP are caught.
+
 
 */
 #define _GNU_SOURCE
@@ -458,6 +461,7 @@ static const char *mount_point = NULL;
 static flag no_syslog = FALSE;
 static char trace_level = 0;
 static flag debug_protocol = FALSE;
+static volatile flag caught_sighup = FALSE;
 static struct initial_symlink_struct
 {
     char *dest;
@@ -1030,13 +1034,13 @@ static void do_servicing (int fd, unsigned long event_mask)
 	SYSLOG (LOG_ERR, "error setting event mask\t%s\n", ERRSTRING);
 	exit (1);
     }
-    while ( ( bytes = read (fd, (char *) &info, sizeof info) ) >= 0 )
+    while (!caught_sighup)
     {
-	if (bytes < 1)
-	{
-	    SYSLOG (LOG_ERR, "devfs closed file!\n");
-	    exit (1);
-	}
+	errno = 0;
+	bytes = read (fd, (char *) &info, sizeof info);
+	if (caught_sighup) break;      /*  Must test for this first     */
+	if (errno == EINTR) continue;  /*  Yes, the order is important  */
+	if (bytes < 1) break;
 	/*  Special trap for "/dev/log" creation  */
 	if (!syslog_is_open && !no_syslog &&
 	    (info.type == DEVFSD_NOTIFY_CREATE) &&
@@ -1052,7 +1056,11 @@ static void do_servicing (int fd, unsigned long event_mask)
 	}
 	service_name (&info);
     }
-    if (errno == EINTR) return;
+    if (caught_sighup)
+    {
+	caught_sighup = FALSE;
+	return;
+    }
     SYSLOG (LOG_ERR, "Error reading control file\t%s\n", ERRSTRING);
     exit (1);
 }   /*  End Function do_servicing  */
@@ -1829,6 +1837,7 @@ static mode_t get_mode (const char *string)
 
 static void sighup_handler (int sig)
 {
+    caught_sighup = TRUE;
     SYSLOG (LOG_INFO, "Caught SIGHUP\n");
 }   /*  End Function sighup_handler  */
 
